@@ -11,18 +11,23 @@ try:
 except:
     import cocos
 
+
 def get_nearest_index(tim_eq, tarr):
-    """Find nearest time idnex for a given time"""
+    """Find nearest time index for a given time"""
 
-    tarr = np.minimum(np.maximum(tarr, tim_eq[0]), tim_eq[-1])
+    tim_eq = np.atleast_1d(np.array(tim_eq, dtype=np.float32))
     if len(tim_eq) == 1:
-        idx = 0
+        unique_idx = [0]
+        index_idx = {0: [0]}
     else:
-        idx = interp1d(tim_eq, np.arange(len(tim_eq)), kind='nearest')(tarr)
-    idx = np.atleast_1d(np.int_(idx))
-    unique_idx = np.unique(idx)
+        idx = np.searchsorted(tim_eq, tarr)
+        unique_idx = np.unique(idx)
+        unique_idx = np.minimum(unique_idx, len(tim_eq) - 1)
+        index_idx = {}
+        for i in unique_idx:
+            (index_idx[i], ) = np.where(idx == i)
 
-    return unique_idx, idx
+    return unique_idx, index_idx
 
 
 def rho2rho(eqm, rho_in, t_in=None, \
@@ -33,8 +38,9 @@ def rho2rho(eqm, rho_in, t_in=None, \
 
     Input
     ----------
-    t_in : float or 1darray
-        time
+    eqm: equilibrium object
+    t_in : float or 1darray or None
+        time (eqm.time if None)
     rho_in : float, ndarray
         radial coordinates, 1D (time constant) or 2D+ (time variable) of size (nt,nx,...)
     coord_in:  str ['rho_pol', 'rho_tor' ,'rho_V', 'r_V', 'Psi','r_a']
@@ -65,7 +71,7 @@ def rho2rho(eqm, rho_in, t_in=None, \
     if coord_out == coord_in: 
         return rho
  
-    unique_idx, idx =  get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx =  get_nearest_index(eqm.time, tarr)
 
     if coord_in in ['rho_pol', 'Psi']:
         label_in = eqm.pfl
@@ -134,11 +140,7 @@ def rho2rho(eqm, rho_in, t_in=None, \
 
         s = UnivariateSpline(rho_in, ratio, w=w, k=4, s=5e-3,ext=3)  #BUG s = 5e-3 can be sometimes too much, sometimes not enought :( 
 
-        if len(idx) == 1:
-            jt = 0
-        else:
-            jt = idx == i
-
+        jt = index_idx[i]
         rho_ = np.copy(rho[jt])
 
         r0_in = 1
@@ -174,15 +176,15 @@ def rz2brzt(eqm, r_in, z_in, t_in=None):
     """calculates Br, Bz, Bt profiles
     Input
     ----------
+    eqm: equilibrium object
     r_in : ndarray
         R coordinates 
         1D, size(nrz_in) or 2D, size (nt, nrz_in)
     z_in : ndarray
         Z coordinates 
         1D, size(nrz_in) or 2D, size (nt, nrz_in)
-    t_in : float or 1darray
-        time
-        If t_in=None, take the whole time array from the shotfile
+    t_in : float or 1darray or None
+        time (eqm.time if None)
 
     Output
     -------
@@ -239,11 +241,11 @@ def rz2brzt(eqm, r_in, z_in, t_in=None):
     psi_fac = psi_sign*psi_2pi
     print('COCOS %d, Sign Bt %d, Sign Ip %d' %(cocos_in, sign_b0, sign_ip))
 
-    unique_idx, idx =  get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx =  get_nearest_index(eqm.time, tarr)
 
     for i in unique_idx:
 
-        jt = (idx == i)
+        jt = index_idx[i]
         Psi = eqm.pfm[:, :, i]
         Br =  np.gradient(Psi, axis=1)/eqm.Rmesh[:, None]
         Bz = -np.gradient(Psi, axis=0)/eqm.Rmesh[:, None]
@@ -268,8 +270,9 @@ def Bmesh(eqm, t_in=None):
     """calculates Br, Bz, Bt profiles
     Input
     ----------
-    t_in : float or 1darray
-        time
+    eqm: equilibrium object
+    t_in : float or 1darray or None
+        time (eqm.time if None)
 
     Output
     -------
@@ -310,14 +313,14 @@ def Bmesh(eqm, t_in=None):
     psi_fac = psi_sign*psi_2pi
     print('COCOS %d' %cocos_in)
 
-    unique_idx, idx =  get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx =  get_nearest_index(eqm.time, tarr)
 
     Br = np.zeros((nt, nr, nz), dtype=np.float32)
     Bz = np.zeros_like(Br)
     Bt = np.zeros_like(Br)
     for i in unique_idx:
 
-        jt = (idx == i)
+        jt = index_idx[i]
         Psi = eqm.pfm[:, :, i]
 # Eq 12 in COCOs paper
         Br[jt] = np.gradient(Psi, axis=1)/eqm.Rmesh[:, None]
@@ -338,8 +341,9 @@ def rz2rho(eqm, r_in, z_in, t_in=None, coord_out='rho_pol', extrapolate=True):
 
     Input
     ----------
-    t_in : float or 1darray
-        time
+    eqm: equilibrium object
+    t_in : float or 1darray or None
+        time (eqm.time if None)
     r_in : ndarray
         R coordinates 
         1D (time constant) or 2D+ (time variable) of size (nt,nx,...)
@@ -390,13 +394,10 @@ def rz2rho(eqm, r_in, z_in, t_in=None, coord_out='rho_pol', extrapolate=True):
     scaling = np.array([dr, dz])
     offset  = np.array([eqm.Rmesh[0], eqm.Zmesh[0]])
     
-    unique_idx, idx =  get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx =  get_nearest_index(eqm.time, tarr)
   
     for i in unique_idx:
-        if len(idx) == 1:
-            jt = 0
-        else:
-            jt = idx == i
+        jt = index_idx[i]
         coords = np.array((r_in[jt], z_in[jt]))
         index = ((coords.T - offset) / scaling).T
         Psi[jt] =  map_coordinates(eqm.pfm[:, :, i], index, mode='nearest',
@@ -416,9 +417,9 @@ def rho2rz(eqm, rho_in, t_in=None, coord_in='rho_pol', all_lines=False):
 
     Input
     ----------
-
+    eqm: equilibrium object
     t_in : float or 1darray
-        time
+        time (eqm.time if None)
     rho_in : 1darray,float
         rho coordinates of the searched flux surfaces
     coord_in: str
@@ -437,26 +438,27 @@ def rho2rz(eqm, rho_in, t_in=None, coord_in='rho_pol', all_lines=False):
         t_in = eqm.time
 
     tarr  = np.atleast_1d(t_in)
-    rhoin = np.atleast_1d(rho_in)
+    rhoin = np.atleast_1d(np.array(rho_in, dtype=np.float32))
 
     rho_in = rho2rho(eqm, rhoin, t_in=t_in, \
              coord_in=coord_in, coord_out='Psi', extrapolate=True )
 
     from matplotlib import _contour
 
+    nt = len(tarr)
     nr = len(eqm.Rmesh)
     nz = len(eqm.Zmesh)
 
     R, Z = np.meshgrid(eqm.Rmesh, eqm.Zmesh)
-    Rsurf = np.empty(len(tarr), dtype='object')
-    zsurf = np.empty(len(tarr), dtype='object')
-    unique_idx, idx =  get_nearest_index(eqm.time, tarr)
+    Rsurf = np.empty(nt, dtype='object')
+    zsurf = np.empty(nt, dtype='object')
+    unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
     for i in unique_idx:
 
-        (jt, ) = np.where(idx == i)
+        jt = index_idx[i]
         Flux = rho_in[jt[0]]
-        
+
 # matplotlib's contour creation
         try:
             cgen = _contour.QuadContourGenerator(R, Z, eqm.pfm[:nr, :nz, i].T, None, True, 0)
@@ -511,10 +513,11 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
 
     Input:
     ----------
+    eqm: equilibrium object
     rho: float
         coordinate of the desired flux surface
     t_in: float or 1darray
-        time point/array for the evaluation
+        time (eqm.time in case of None)
     r_in: float
         R position of the point
     z_in: float
@@ -526,9 +529,9 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
 
     Output:
     ----------
-    Rout: 3darray size(nt, nx, 2)
+    Rout: 3darray size(nt, 2)
         R-position of intersections
-    zout: 3darray size(nt, nx, 2)
+    zout: 3darray size(nt, 2)
         z-position of intersections
     """
 
@@ -540,7 +543,7 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
     r_in = np.float32(r_in)
     z_in = np.float32(z_in)
 
-    unique_idx, idx = get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
     line_m = 3. # line length: 6 m
     n_line = int(200*line_m) + 1 # 1 cm, but then there's interpolation!
@@ -560,10 +563,7 @@ def cross_surf(eqm, rho=1., r_in=1.65, z_in=0, theta_in=0, t_in=None, coord_in='
     Rout = np.zeros((nt_in, 2), dtype=np.float32)
     zout = np.zeros((nt_in, 2), dtype=np.float32)
     for i, ii in enumerate(unique_idx):
-        if len(idx) == 1:
-            jt = 0
-        else:
-            jt = idx == i
+        jt = index_idx[i]
         ind_gt = (rho_line[i] > rho)
         ind_cross = [j for j, x in enumerate(np.diff(ind_gt)) if x]
         if len(ind_cross) > 2:
@@ -595,8 +595,9 @@ def cross_sep(eqm, r_in=1.65, z_in=0, theta_in=0, t_in=None):
 
     Input:
     ----------
+    eqm: equilibrium object
     t_in: float or 1darray
-        time point/array for the evaluation
+        time (eqm.time in case of None)
     r_in: float
         R position of the point
     z_in: float
@@ -619,7 +620,7 @@ def cross_sep(eqm, r_in=1.65, z_in=0, theta_in=0, t_in=None):
     r_in = np.float32(r_in)
     z_in = np.float32(z_in)
 
-    unique_idx, idx = get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
     line_m = 3. # line length: 6 m
     n_line = int(200*line_m) + 1 # 1 cm, but then there's interpolation!
@@ -638,10 +639,7 @@ def cross_sep(eqm, r_in=1.65, z_in=0, theta_in=0, t_in=None):
     Rout = np.zeros((nt_in, 2), dtype=np.float32)
     zout = np.zeros((nt_in, 2), dtype=np.float32)
     for i, ii in enumerate(unique_idx):
-        if len(idx) == 1:
-            jt = 0
-        else:
-            jt = idx == i
+        jt = index_idx[i]
         ind_gt = (psi_line[i] > psi_sep[ii])
         ind_cross = [j for j, x in enumerate(np.diff(ind_gt)) if x]
         if len(ind_cross) > 2:
@@ -679,11 +677,9 @@ def rhoTheta2rz(eqm, rho, theta_in, t_in=None, coord_in='rho_pol', n_line=201):
     rho: float or 1D array (nr) or nD array (nt,nr,...)
         coordinate of the desired flux surface inside LCFS!
     t_in: float or 1darray
-        time point/array for the evaluation
-
+        time (eqm.time in case of None)
     theta_in: float or 1D array n_theta
         angle of the straight line with respect to horizontal-outward, in radians!!
-        
     coord_in:  str ['rho_pol', 'rho_tor' ,'rho_V', 'r_V', 'Psi','r_a']
         input coordinate label
 
@@ -706,7 +702,7 @@ def rhoTheta2rz(eqm, rho, theta_in, t_in=None, coord_in='rho_pol', n_line=201):
     theta_in = np.atleast_1d(theta_in)[:, None]
     ntheta = len(theta_in)
 
-    unique_idx, idx = get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
 #        n_line = 201 <=> 5 mm, but then there's interpolation!
 
@@ -738,7 +734,7 @@ def rhoTheta2rz(eqm, rho, theta_in, t_in=None, coord_in='rho_pol', n_line=201):
         rho_line[:,:,0] = 0
 
     for i, ii in enumerate(unique_idx):
-        jt = idx == ii
+        jt = index_idx[ii]
         for k in range(ntheta):
             rho_lin = rho_line[i, k]
             (tmp, ) = np.where(np.diff(rho_lin) > 0)
@@ -762,6 +758,7 @@ def mag_theta_star(eqm, t_in, n_rho=400, n_theta=200, rz_grid=False ):
 
     Input:
     ----------
+    eqm: equilibrium object
     t_in: float 
         time point for the evaluation
     n_rho: int
@@ -834,20 +831,40 @@ def mag_theta_star(eqm, t_in, n_rho=400, n_theta=200, rz_grid=False ):
     return rgrid, zgrid, theta_star
 
 
-def get_q_surf(eqm, qvalue=1, t_in=None, coord_out='rho_tor'):
+def get_q_surf(eqm, qvalue=1., t_in=None, coord_out='rho_tor'):
+    """
+    Computes the most external magnetic-surface with a given q-value
+    If the value is not found, it returns np.nan at that time point
 
+    Input:
+    -----------
+    eqm: equilibrium object
+    qvalue: float > 0
+        Desired q-value for magnetic surface
+    t_in : float or 1darray or None
+        time (eqm.time if None)
+    coord_out: str ['rho_pol', 'rho_tor' ,'rho_V', 'r_V', 'Psi','r_a']
+        output coordinate label
+
+    Output:
+    -----------
+    Float or list rho_surf, same size as t_in
+    """
+
+    qvalue = float(qvalue)
     if t_in is None:
         t_in = eqm.time
     tarr = np.atleast_1d(t_in)
-    unique_idx, idx = get_nearest_index(eqm.time, tarr)
+    unique_idx, index_idx = get_nearest_index(eqm.time, tarr)
 
-    rho_surf = []
-    rho_q = rho2rho(eqm, eqm.pfl[unique_idx], t_in=t_in, coord_in='Psi', coord_out=coord_out)
-
-    for i in unique_idx:
-        (jt, ) = np.where(idx == i)[0]
+    rho_surf = np.zeros_like(tarr)
+    rho_q = rho2rho(eqm, eqm.pfl[unique_idx], t_in=eqm.time[unique_idx], coord_in='Psi', coord_out=coord_out)
+    qprof = np.abs(eqm.q)
+    for ienum, i in enumerate(unique_idx):
         (nrho, ) = eqm.lpfp[i]
-        q_surf = np.interp(qvalue, np.abs(eqm.q[i, :nrho]), rho_q[jt, :nrho], left=np.nan, right=np.nan)
-        rho_surf.append(q_surf)
+        jmin = np.argmin(qprof[i, :nrho])
+        q_surf = np.interp(qvalue, qprof[i, jmin:nrho], rho_q[ienum, jmin:nrho], left=np.nan, right=np.nan)
+        for j in index_idx[i]:
+            rho_surf[j] = q_surf
 
     return np.squeeze(rho_surf)
